
/**
 * プラグインのタイプ
 * request-hookは通信内容をフックして処理を行います
 */
module.exports.type = 'request-hook'

/**
 * プラグインの情報
 */
module.exports.meta = {
  name: '5ch_subject_sort',
  description: '5ch_subject_sort',
  version: '1.0.0',
  needVersion: '0.24.12'
}

/**
 * 
 * 
 * @module afterRequest URLリクエスト後のヘッダや内容を変更します。
 * @param {Object} tools - データの保存などで利用できるツール
 * @param {Conf} tools.conf
 * @param {SimpleFetch} tools.simplefetch
 * @param {Log} tools.log
 * @param {{ [key: string]: string }} tools.siki_values - workspaceidやuser_dir,log_dir等の値が入っています
 * @param {string} url - リクエストされたURL
 * @param {number} statusCode - HTTPステータスコード
 * @param {{ [key: string]: string }} response_headers - サーバーから帰って来たhttpヘッダー 全て小文字となります
 * @param {string} response_body - 受け取ったレスポンス内容 レスポンスのエンコーディングに関係なくユニコードに変換済み
 * @return {{statusCode?: number, response_headers?: { [key: string]: string }, response_body?: string}} - レスポンスヘッダと内容を返してください 返さない場合は本来の内容で処理が進みます
 * 
 * @example - 受け取った内容からストレージへ保存
 * const afterRequest = async (tools, url, response_headers, response_body) => {
 *   const { conf } = tools
 *   if(url.match(/example\.com/)) {
 *     conf.set('x', response_body)
 *   }
 *   return { response_headers, response_body }
 * }
 */
module.exports.afterRequest = async (tools, url, statusCode, response_headers, response_body) => {
  const { conf, simplefetch, log, siki_values } = tools
  try {
    if (url.match(/(?:5ch\.net|bbspink\.com)\/\w+\/lastmodify\.txt$/)) {
      let new_lastmodify = []
      let lm = response_body.split(/\n/).reduce((acc, line) => {
        acc[line.split("<>", 1)[0]] = line
        return acc
      }, {})
      let { body: subject } = await simplefetch(url.replace(/lastmodify\.txt/, 'subject.txt'))
      for (let s of subject.split(/\n/)) {
        let [threadkey] = s.split("<>")
        if (threadkey && lm[threadkey]) {
          new_lastmodify.push(lm[threadkey])
        }
      }
      response_body = new_lastmodify.join("\n")
    }
  } catch (e) {
    throw e
  }
  return { statusCode, response_headers, response_body }
}
